using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using PhoenixDLL.Class;
using Utils.Logs;
using PhoenixPrescription.Class;
using System.Reflection;

namespace PhoenixCards.GUI
{
    /// <summary>
    /// Prescriptions manager
    /// </summary>
    public partial class fr_Prescriptions : Form
    {
        #region Variables
        /// <summary>
        /// Patient manager
        /// </summary>
        private PatientManager vPatientManager;
        /// <summary>
        /// SQL Manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Indicates whether is the first set up of dgv_Selection
        /// </summary>
        private bool vIsFirstSetup = true;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_Prescriptions()
        {
            InitializeComponent();
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the information about the patient
        /// </summary>
        public PatientManager PatientManager { get { return vPatientManager; } set { vPatientManager = value; } }
        #endregion

        #region Events
        /// <summary>
        /// The frame is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
            ProcessSearchPrescriptions();
        }
        /// <summary>
        /// Search a set of prescriptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Search(object sender, EventArgs e)
        {
            ProcessSearchPrescriptions();
        }
        /// <summary>
        /// Clear the selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_SelectionMode(object sender, EventArgs e)
        {
            dgv_Selection.Rows.Clear();
            this.ProcessSearchPrescriptions();
        }
        /// <summary>
        /// Send the selection to Ms Word
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_SendToWord(object sender, EventArgs e)
        {
            try
            {
                Word.Application vMsWord;
                object vWordMissing = Missing.Value;

                vMsWord = new Word.Application();
                vMsWord.Visible = true;
                Word.Document nvDoc;
                object Template = RegistryManager.Path + @"\Prescription.dot";
                nvDoc = vMsWord.Documents.Add(ref Template, ref vWordMissing, ref vWordMissing, ref vWordMissing);
                nvDoc.FormFields.Item(ref TextShaper.Fields[1]).Result = Options.Instance.PrintPrescriptionHeader;
                nvDoc.FormFields.Item(ref TextShaper.Fields[2]).Result = this.vPatientManager.ToString();
                TextShaper ts = new TextShaper();
                Prescription p = new Prescription();
                for (int i = 0; i < dgv_Selection.Rows.Count; i++)
                {
                    p.Drug = DrugsManager.Create((int)dgv_Selection.Rows[i].Cells["m.id_medicament"].Value);
                    p.Remark = (string)dgv_Selection.Rows[i].Cells["p.Rmq"].Value;
                    ts.Prescriptions.Add(p);
                }
                DataObject dob = new DataObject(DataFormats.Rtf, ts.RTF);
                Clipboard.SetDataObject(dob, false);
                nvDoc.Bookmarks.Item(ref TextShaper.Fields[3]).Range.Paste();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible d'ouvrir le document MSWord!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }

        #region Drag'n'Drop
        /// <summary>
        /// Mouse click which starts the drag'n'drop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Prescription_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                DataGridView.HitTestInfo HitTest = dgv_Prescriptions.HitTest(e.X, e.Y);
                DataGridViewSelectedRowCollection dgvrc;

                if (HitTest.RowIndex >= 0 && dgv_Prescriptions.SelectedRows.Count > 0)
                {
                    #region Select clicked row
                    dgv_Prescriptions.SelectedRows[0].Selected = false;
                    dgv_Prescriptions.Rows[HitTest.RowIndex].Selected = true;
                    #endregion

                    dgvrc = dgv_Prescriptions.SelectedRows;
                    dgv_Prescriptions.DoDragDrop(dgvrc, DragDropEffects.Copy);
                }
            }
        }
        /// <summary>
        /// Enter drag from Prescription DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Selection_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Copy;
        }
        /// <summary>
        /// DragDrop in Selection DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Selection_DragDrop(object sender, DragEventArgs e)
        {
            try
            {
                if (!e.Data.GetDataPresent(typeof(DataGridViewSelectedRowCollection))) return;
                DataGridViewSelectedRowCollection dgvrc =
                    (DataGridViewSelectedRowCollection)e.Data.GetData(typeof(DataGridViewSelectedRowCollection));

                for (int i = 0; i < dgvrc.Count; i++)
                {
                    DataGridViewRow row = (DataGridViewRow)dgvrc[i].Clone();
                    for (Int32 index = 0; index < row.Cells.Count; index++)
                    {
                        row.Cells[index].Value = dgvrc[i].Cells[index].Value;
                    }
                    dgv_Selection.Rows.Add(row);

                    #region Remove rows from dgv_Prescription
                    for (int j = 0; j < dgv_Prescriptions.SelectedRows.Count; j++)
                    {
                        dgv_Prescriptions.Rows.Remove(dgv_Prescriptions.SelectedRows[j]);
                    }
                    #endregion
                }
                SetDgv();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est apparue pendant le Drag'n'Drop!", "Erreur",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        #endregion
        #endregion

        #region Methods
        /// <summary>
        /// Process a search of prescriptions
        /// </summary>
        private void ProcessSearchPrescriptions()
        {
            try
            {
                #region Fill dgv_Prescription
                dgv_Prescriptions.DataSource = null;
                dgv_Prescriptions.Rows.Clear();
                string stDateFrom = dtp_From.Value.Day + "/" + dtp_From.Value.Month + "/" + dtp_From.Value.Year;
                string stDateUntill = dtp_Until.Value.Day + "/" + dtp_Until.Value.Month + "/" + dtp_Until.Value.Year;
                SqlManager.SqlQuery = "SELECT * FROM Prescription p, Medicament m WHERE p.id_Patient = " + vPatientManager.IDPatient
                    + " AND p.Date_P >= FORMAT(\"" + stDateFrom + "\", \"dd/mm/yyyy\") AND p.Date_P <= FORMAT(\""
                    + stDateUntill + "\", \"dd/mm/yyyy\") AND p.id_medicament = m.id_medicament;";
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);
                dgv_Prescriptions.DataSource = ds.Tables[0];
                #endregion

                #region Clear dgv_Selection
                if (vIsFirstSetup)
                {
                    vIsFirstSetup = false;
                    dgv_Selection.Columns.Clear();
                    for (int i = 0; i < dgv_Prescriptions.Columns.Count; i++)
                        dgv_Selection.Columns.Add((DataGridViewColumn)dgv_Prescriptions.Columns[i].Clone());
                }
                #endregion

                SetDgv();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue lors de la recherche!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// Setup the display of the DGV
        /// </summary>
        private void SetDgv()
        {
            #region Set dgv_Prescription
            for (int i = 0; i < dgv_Prescriptions.Columns.Count; i++)
                dgv_Prescriptions.Columns[i].Visible = false;

            dgv_Prescriptions.Columns["Date_p"].Visible = true;
            dgv_Prescriptions.Columns["Date_p"].DisplayIndex = 0;
            dgv_Prescriptions.Columns["Date_p"].HeaderText = "Date";
            dgv_Prescriptions.Columns["Date_P"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;

            dgv_Prescriptions.Columns["Nom"].Visible = true;
            dgv_Prescriptions.Columns["Nom"].DisplayIndex = 1;
            dgv_Prescriptions.Columns["Nom"].HeaderText = "Nom";
            dgv_Prescriptions.Columns["Nom"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;

            dgv_Prescriptions.Columns["p.Rmq"].Visible = true;
            dgv_Prescriptions.Columns["p.Rmq"].DisplayIndex = 2;
            dgv_Prescriptions.Columns["p.Rmq"].HeaderText = "Remarques";
            dgv_Prescriptions.Columns["p.Rmq"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            #endregion

            #region Set dgv_Selection
            for (int i = 0; i < dgv_Selection.Columns.Count; i++)
                dgv_Selection.Columns[i].Visible = false;

            dgv_Selection.Columns["Date_p"].Visible = true;
            dgv_Selection.Columns["Date_p"].DisplayIndex = 0;
            dgv_Selection.Columns["Date_p"].HeaderText = "Date";
            dgv_Selection.Columns["Date_P"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;

            dgv_Selection.Columns["Nom"].Visible = true;
            dgv_Selection.Columns["Nom"].DisplayIndex = 1;
            dgv_Selection.Columns["Nom"].HeaderText = "Nom";
            dgv_Selection.Columns["Nom"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;

            dgv_Selection.Columns["p.Rmq"].Visible = true;
            dgv_Selection.Columns["p.Rmq"].DisplayIndex = 2;
            dgv_Selection.Columns["p.Rmq"].HeaderText = "Remarques";
            dgv_Selection.Columns["p.Rmq"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            #endregion
        }
        #endregion
    }
}